#!/usr/bin/perl
#########################################################################
#FLEMM-v2 -- French Lemmatizer : Lemmatisation du franais  partir de  # 
#corpus tiquets - Version 2						            #
#Copyright (C) 1999 (NAMER Fiammetta)					      #
#########################################################################

# Programme d'initialisation du lemmatiseur
# Appelle le programme de preparation qui va:
# - appeller le lemmatiseur,
# - formater le contenu selon les notations de l'utilisateur

# Pour utiliser les options dans la commande perl
use Getopt::Long;


package main;

# La variable Globale $directory est dfinie dans 
# le programme principal et porte sur tous les modules appels

$directory="";

&main;

sub usage {
   print STDERR "Usage: Les arguments notes entre [ ] sont optionnels.
                 Par defaut, 
                    - en l'absence de --repertoire, le repertoire 
                      d'installation du programme est /usr/share/flemm,
                    - en l'absence de --sortie, le rsultat est affiche dans le
                      fichier AdresseGlobaleDuFichierDentree.lemm
                    - l'option --pgm ne sert que lorsque FLEMM est utilise en 
                      aval d'un autre programme
                    - en l'absence de --log, pas de fichier contenant la 
                      correction des (eventuelles) erreurs d'etiquetage
                    - en l'absence de --tagger, l'etiqueteur par defaut est 
                      \"Brill\". Les seules valeurs valides pour l'etiqueteur
                      sont actuellement Brill ou Treetagger

         perl init_flemm.perl --entree AdresseGlobaleDuFichierDentree 
                             [--repertoire AdresseGlobaleDuRepertoireDeTravail]
                             [--sortie AdresseGlobaleDuFichierDeSortie]
                             [--pgm ProgrammeAppelant --pgm_arg SesArguments]
                             [--log]
                             [--tagger NomDeLEtiqueteur]\n\n";
}
                                     
# Programme principal

sub main {
    
    my ($filename, $file_output,$pgm, $pgm_arg,$file_log,$tagger)=("","","","",0,"");
    
    my($entree)="";
    
    my($a,$nb_brill);
    my(@tab);
    my($LEMM);
    
    my($result);
    my($res)="";

    $result=&GetOptions
	(
	 "entree=s" => \$filename, # adresse globale du fichier 
	                           # d'entree
	 "sortie:s" => \$file_output, # adresse globale du 
			              # fichier de sortie 
	                              # (optionnelle);
	 "repertoire:s" => \$directory, # ou se trouve le prog
	 "pgm:s" => \$pgm, # programme appelant
	 "pgm_arg:s" => \$pgm_arg, # argument du programme appelant
	 "log!"   => \$file_log,   # 0 ou 1 (--log ou --nolog)
	 "tagger:s" => \$tagger); # Tagger utilis : Brill ou Treetagger (opt.)
    

    if ($filename eq "") {
	&usage();
      #print "\nIl faut definir le fichier d'entree (option --entree)\n";
    }
    
    #elsif ($pgm eq "") {
    #	print OUT1 "\nIl faut definir le programme appelant (option --pgm)\n";
    #}
    
    # Si l'option programme appelant est dfini, alors 
    # son argument doit l'tre aussi.

    elsif (($pgm ne "") && ($pgm_arg eq "")) {
	print "\nIl faut definir l'argument de $pgm (option --pgm_arg)\n";
    }
    
    else {

    if ($directory eq "") {
	$directory = "/usr/share/flemm";
	print "\nValeur par defaut de l'adresse d'installation de Flemm: $directory\n";
    }

	print  "\nFichier d'entree : $filename\n";
	print  "\nAdresse d'installation de Flemm: $directory\n";

      if (($tagger eq "")|| ($tagger =~ /[Bb]rill/)) {
	    print "\nL'etiqueteur est Brill\n";
	    require "$directory/entrees_sorties";
          $LEMM = new entrees_sorties("$directory/EXCEP");

	}
	else {
           print "\nL'etiqueteur est Treetagger\n";
	     require "$directory/entrees_sortiesTT";
           $LEMM = new entrees_sortiesTT("$directory/EXCEP");

      }
	
	if ($file_output eq "") {
		$file_output = $filename.".lemm";
		print "\nPar defaut, le fichier de sortie s'appelle : $file_output\n";
      }
	else {
		print "\nFichier de sortie : $file_output\n";
      }		
	open(INPUT,"$filename") || die "pas de fichier $!";
      open(OUT0,">$file_output") || die "$!";
	
        if ($file_log==1) {
	    print "\nLes fichiers log s'appellent ";
	    
	    $file_log = $filename;
	    print "$file_log.etiq (etiquetage) et $file_log.seg (segmentation)\n";

      # Effacer le contenu pralable ventuel.   

	unlink "$file_log.etiq" || warn "$!";
	unlink "$file_log.seg" || warn "$!";


	}
	else {
	    $file_log="";
	}
	
	while (<INPUT>) {	
	    
	    chop; 
	    $entree = $_;
	    
          if ($tagger eq "") { # Si le tagger est Brill :

	       # Correction de  "./.</ph=1></</txt=000028>" en  "./.</ph=1></txt=000028>"
	       $entree =~ s/\<\/\</\</g;
	    
	       # Espaage de  "./.</ph=1></txt=000028>" en  "./.</ph=1> </txt=000028>"
	       $entree =~ s/\>/\> /g;
	    
	       # Espaage de  "BLA</ph=1>" en  "BLA </ph=1>"
	       $entree =~ s/([^ ])\</$1 \</g;
	       $entree =~ s/\s+/ /g;
	       @tab = split(' ',$entree);
	    
	       foreach $a (@tab) {
		
		    $nb_brill="";
		
		    # Conservation du nombre calcul par BRill.
		
		    if ($a =~ /:(.*)$/) {
		        $nb_brill = $1;
		    }
		
		    $a =~ s/(.):..$/$1/;
		
		    if (($a =~ /^\</)||($a =~ /^.\/.$/)) {
		        $res = $a;
		    }
		    else {
		        $res = $LEMM->identifie($a,$nb_brill, $file_log);
		    }
		
		    # Pour texte:
		    print OUT0 "$res";
		    if ($res !~ /\>$/) {
		         print OUT0 " ";
		    }
	       }
	      print OUT0 "\n";
        }
        else {  # Sinon, le tagger est Treetagger
             $entree =~ /^(.*)\t(.*)\t.*$/;
             $a = $1."/".$2;
             $res = $LEMM->identifie($a,$file_log);
		 $res =~s/\//\t/g;
             print OUT0 "$res\n";

        }
	    next;
	}
    }
   
    close(INPUT);
    close(OUT0);

    # Selon que le programme est appel par une ligne de commande ou par un 
    # autre programme, on sort ou on rappelle le programme appelant.

    if ($pgm eq "") {
	exit(0);
	}
    else {
    	exec $pgm, $pgm_arg;
    }
}




